/* This file is part of the KDE Project
   Copyright (c) 2004 Kévin Ottens <ervin ipsquad net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef REMOTEDIRNOTIFY_H
#define REMOTEDIRNOTIFY_H

#include <kurl.h>
#include <QtCore/QObject>

class RemoteDirNotify : public QObject
{
Q_OBJECT

public:
	RemoteDirNotify();

private slots:
	void FilesAdded (const QString &directory);
	void FilesRemoved (const QStringList &fileList);
	void FilesChanged (const QStringList &fileList);

private:
	KUrl toRemoteURL(const KUrl &url);
	KUrl::List toRemoteURLList(const KUrl::List &list);
	KUrl m_baseURL;
};

#endif
